package model;

import java.util.ArrayList;

public class Item {
    private int id;
    private String name;
    private Money price;
    private final ItemCategory category;
    private ArrayList<AddOn> addOns;

    public Item(int id, String name, Money price, ItemCategory category, ArrayList<AddOn> addOns){
        this.id = id;
        this.name = name;
        this.price = price;
        this.category = category;
        this.addOns = addOns;
    }
    public Item(int id, String name, Money price, ItemCategory category){
        this.id = id;
        this.name = name;
        this.price = price;
        this.category = category;
        addOns = new ArrayList<>();
    }
    public Item(String name, Money price, ItemCategory category){
        // id already zero
        this.name = name;
        this.price = price;
        this.category = category;
        addOns = new ArrayList<>();
    }

    public Money priceWithAddOns() {
        Money total = price;
        for (int i = 0; i < addOns.size(); i++) {
            total = total.add(addOns.get(i).getPrice());
        }
        return total;
    }

    public int getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public Money getPrice() {
        return price;
    }

    public ItemCategory getCategory() {
        return category;
    }

    public ArrayList<AddOn> getAddOns() {
        return addOns;
    }

    public void addAddOn(AddOn addOn){
        addOns.add(addOn);
    }

    public void setName(String name){
        this.name = name;
    }

    @Override
    public String toString(){
        String rtn = name + " " + price;
        if(!addOns.isEmpty()){
            rtn = rtn + " (" + priceWithAddOns() + ")";
        }

        for (int i = 0; i < addOns.size(); i++) {
            rtn = rtn + "\n    + " + addOns.get(i);
        }

        return rtn;
    }
}
