package model;

public class Money {
    private final Currency currency;
    private int wholeValue;
    private int decimalValue;

    public Money(int wholeValue, int decimalValue){
        if(decimalValue > 99){
            throw new RuntimeException("decimal value of money must be less than 99");
        }
        currency = Currency.USD;
        this.wholeValue = wholeValue;
        this.decimalValue = decimalValue;
    }

    public Money(Currency currency, int wholeValue, int decimalValue){
        if(decimalValue > 99){
            throw new RuntimeException("decimal value of money must be less than 99");
        }
        this.currency = currency;
        this.wholeValue = wholeValue;
        this.decimalValue = decimalValue;
    }

    public Money(double value){
        this.currency = Currency.USD;
        int whole = (int) value;
        double decimal = (value - whole)*100;
        this.wholeValue = whole;
        this.decimalValue = (int) Math.round(decimal);
    }

    public Money(double value, Currency currency){
        this.currency = currency;
        int whole = (int) value;
        double decimal = (value - whole)*100;
        this.wholeValue = whole;
        this.decimalValue = (int) Math.round(decimal);
    }

    public static Money tax(Money total, double tax, Currency currency) {
        double grandTotal = total.castToDouble() * (1 + tax);
        return parse(currency, grandTotal);
    }

    public Money add(Money price) {
        if(this.currency != price.currency){
            throw new RuntimeException("Currency does not match");
        }
        double tmp = price.castToDouble() + this.castToDouble();
        return new Money(tmp, this.currency);
    }

    public Money subtract(Money price){
        if(this.currency != price.currency){
            throw new RuntimeException("Currency does not match");
        }
        double tmp = this.castToDouble() - price.castToDouble();
        return new Money(tmp, this.currency);
    }

    public Currency getCurrency() {
        return currency;
    }

    public int getWholeValue() {
        return wholeValue;
    }

    public int getDecimalValue() {
        return decimalValue;
    }

    public double castToDouble(){
        return this.wholeValue + this.decimalValue * 0.01;
    }

    public static Money parse(Currency currency, double value){
        return new Money(value, currency);
    }

    @Override
    public String toString(){
        String sym = "$";
        if(currency != Currency.USD){
            sym = currency.toString() + "-";
        }

        if(decimalValue < 10){
            return sym + wholeValue + ".0" + decimalValue;
        } else {
            return sym + wholeValue + "." + decimalValue;
        }
    }
}
