package model;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class Order {
    private int id;
    private ArrayList<Item> items;
    private boolean refunded;
    private Date dateTime;

    private final ItemCategory[] taxables = {ItemCategory.MERCHANDISE, ItemCategory.FOOD};


    public Order(int id, ArrayList<Item> items, boolean refunded, Date dateTime){
        this.id = id;
        this.items = items;
        this.refunded = refunded;
        this.dateTime = dateTime;
    }

    public Order(){
        // id already zero
        items = new ArrayList<>();
        // refunded already false
        this.dateTime = new Date();
    }

    public Order(int id){
        this.id = id;
        items = new ArrayList<>();
        // refunded already false
        this.dateTime = new Date();
    }

    public void addItem(Item item){
        items.add(item);
    }

    public String getDateTimeAsString(){
        SimpleDateFormat ft = new SimpleDateFormat("E yyyy.MM.dd 'at' hh:mm:ss a zzz");
        return ft.format(dateTime);
    }

    public void setRefund(boolean refunded){
        this.refunded = refunded;
    }

    public void setDate(Date date){
        this.dateTime = date;
    }

    public void setTime(long time){
        this.dateTime.setTime(time);
    }

    public void clear(){
        items.clear();
    }

    public int getId() {
        return id;
    }

    public ArrayList<Item> getItems() {
        return items;
    }

    public void setId(int id){
        this.id = id;
    }

    public boolean isRefunded() {
        return refunded;
    }

    public Date getDateTime() {
        return dateTime;
    }

    public Money getTaxable(){
        Money m = new Money(0.0);
        for (int i = 0; i < items.size(); i++) {
            Item item = items.get(i);
            if(isTaxable(item.getCategory())){
                m = m.add(items.get(i).priceWithAddOns());
            }
        }
        return m;
    }

    public boolean isTaxable(ItemCategory cat){
        for (int i = 0; i < taxables.length; i++) {
            if(cat == taxables[i]){
                return true;
            }
        }
        return false;
    }

    public Money getTotal(){
        Money m = new Money(0.0);
        for (int i = 0; i < items.size(); i++) {
            m = m.add(items.get(i).priceWithAddOns());
        }
        return m;
    }

    @Override
    public String toString(){
        String rtn = "Order: " + id + " (" + getTotal().toString() + ")\n";
        for (int i = 0; i < items.size(); i++) {
            rtn = rtn + "  * " + items.get(i).toString() + "\n";
        }
        return rtn;
    }

}
