package model;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Random;

public class Register {
    private ArrayList<Order> orders;
    private SQLConnection dataEngine;
    private Order activeOrder;

    private static final Random rand = new Random();

    public Register(){
        orders = new ArrayList<>();
        dataEngine = new SQLConnection();
        activeOrder = new Order();
    }

    public void setLocalOrders(ArrayList<Order> orders){
        this.orders = orders;
    }

    public Order getLocalOrder(int i){
        return orders.get(i);
    }

    public void addOrder(Order o){
        boolean match = false;
        for (int i = 0; i < orders.size(); i++) {
            if(orders.get(i).getId() == o.getId()){
                match = true;
                Date now = new Date();
                orders.get(i).setDate(now);
                break;
            }
        }
        if(!match) {
            try {
                o.setId(dataEngine.maxID("orders") + 1);
                dataEngine.recordOrder(o);
                orders.add(o);
            } catch (SQLException e){
                throw new RuntimeException(e);
            }
        }
    }

    public int getMaxLocalId(){
        int max = 1;
        for (int i = 0; i < orders.size(); i++) {
            if (orders.get(i).getId() > max){
                max = orders.get(i).getId();
            }
        }
        return max;
    }

    public void deleteOrder(int id){
        boolean found = false;
        for (int i = 0; i < orders.size(); i++) {
            if(orders.get(i).getId() == id){
                found = true;
                dataEngine.deleteOrderById(orders.get(i).getId());
                orders.remove(i);
                break;
            }
        }
        if(!found){
            throw new RuntimeException("Order " + id + " was not found.");
        }
    }

    public void refundOrder(int orderID){
        dataEngine.refundOrder(orderID);
    }

    public Order getActiveOrder(){
        return activeOrder;
    }

    public int getOrdersSize(){
        return orders.size();
    }

    public ArrayList<Order> getOrders() {
        return orders;
    }

    public Order getOrderById(int id){
        return dataEngine.getOrderById(id);
    }

    public static Order randomOrder(){
        Order o = new Order(0);
        Money fryPrice = new Money(2,0);
        //public Item(String name, Money price, ItemCategory category)
        Item fry = new Item("Fries", fryPrice, ItemCategory.FOOD);

        int numBurgers = rand.nextInt(6) + 1;

        for (int i = 0; i < numBurgers; i++) {
            if(rand.nextBoolean()){
                o.addItem(fry);
            }
            o.addItem(randomBurger());
        }

        return o;
    }

    private static Item randomBurger() {
        Money five = new Money(5,0);
        Money tenC = new Money(0,10);
        Money oneD = new Money(1,0);
        Money zero = new Money(0,0);
        Item b = new Item("Burger", five, ItemCategory.FOOD);


        // AddOn(String name, Money price)
        AddOn ketchup = new AddOn("Ketchup", zero);
        AddOn cheese = new AddOn("Cheese", tenC);
        AddOn mayo = new AddOn("Mayo", zero);
        AddOn mustard = new AddOn("Mustard", zero);
        AddOn lettuce = new AddOn("Lettuce", zero);
        AddOn pickle = new AddOn("Pickle", zero);
        AddOn onion = new AddOn("Onion", zero);
        AddOn noBun = new AddOn("NO-BUN", zero);
        AddOn bacon = new AddOn("Bacon", zero);
        ArrayList<AddOn> addOns = new ArrayList<>();
        addOns.add(ketchup);
        addOns.add(cheese);
        addOns.add(mayo);
        addOns.add(mustard);
        addOns.add(lettuce);
        addOns.add(pickle);
        addOns.add(onion);
        addOns.add(noBun);
        addOns.add(bacon);

        int numAddOns = rand.nextInt(5);

        for (int i = 0; i < numAddOns; i++) {
            int next = rand.nextInt(addOns.size());
            b.addAddOn(addOns.get(next));
            addOns.remove(next);
        }

        return b;
    }

    public void setActiveOrder(Order o) {
        this.activeOrder = o;
    }
}
