package model;

import java.sql.*;
import java.util.Timer;
import java.util.TimerTask;

public class SQLConnection {
    private Connection conn;
    private Statement stmt;
    private final Currency currency = Currency.USD;

    private int secondsIdle;
    private static final int timeout = 10; // five minutes (300 seconds)
    private boolean terminated = false;


    // Local Database
    static final String URL = "jdbc:mysql://localhost:3306/burgerShopDatabase";
    static final String user = "root";
    static final String password = "";


    TimerTask task = new TimerTask() {
        @Override
        public void run() {
            while (true){
                try {
                    Thread.sleep(5000); // check every five seconds
                    secondsIdle = secondsIdle + 5; // add five seconds !!! MUST MATCH ABOVE TIME !!!
                    if(secondsIdle > timeout && !conn.isClosed()){
                        conn.close();
                        System.out.println("The database connection (" + user + "@" + URL + ") timed out.");
                    }
                } catch (InterruptedException | SQLException e) {
                    throw new RuntimeException(e);
                }
            }
        }
    };



    public SQLConnection(){
        try {
            conn = DriverManager.getConnection(URL, user, password);
            stmt = conn.createStatement();
            secondsIdle = 0;

            Timer timer = new Timer("Timer");

            long delay = 10000;
            timer.schedule(task, delay);

        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }


    // Use variable database
    public SQLConnection(String URL, String user, String password){
        try {
            conn = DriverManager.getConnection(URL, user, password);
            stmt = conn.createStatement();
            secondsIdle = 0;

            Timer timer = new Timer("Timer");

            long delay = 10000;
            timer.schedule(task, delay);
        } catch (SQLException e){
            throw new RuntimeException(e);
        }
    }



    ///////////////////////////////// METHODS /////////////////////////////////////////////////


    public void updateConnection(){
        if(terminated){
            throw new RuntimeException("This connection has been terminated");
        }
        try {
            if (conn.isClosed()) {
                conn = DriverManager.getConnection(URL, user, password);
                stmt = conn.createStatement();
            }
            secondsIdle = 0;
        } catch (SQLException e){
            throw new RuntimeException(e);
        }
    }



    public void deleteOrderById(int id){
        updateConnection();
        try{
            stmt.executeUpdate("DELETE FROM orders WHERE id=" + id + ";");
            stmt.executeUpdate("DELETE FROM addOns WHERE parentID=" + id + ";");
        } catch (SQLException e){
            throw new RuntimeException(e);
        }
    }

    public Order getOrderById(int id){
        updateConnection();
        try {
            Order rtn = new Order(id);

            ResultSet rs = stmt.executeQuery("SELECT * FROM orders WHERE id=" + id + ";");

            if(rs.next()){
                rtn.setRefund(rs.getBoolean("refunded"));
                //////////////////////////// FIX DATETIME ///////////////////////////////////////////////
                rtn.setTime(rs.getTime("date").getTime());
                rtn.setDate(rs.getDate("date"));
                Statement itemsStmt = conn.createStatement();
                ResultSet itemsRs = itemsStmt.executeQuery("SELECT * FROM items WHERE parentID=" + id + ";");
                while (itemsRs.next()){
                    Item tmpItem = new Item(
                            // Item(int id, String name, Money price, ItemCategory category)
                            itemsRs.getInt("id"),
                            itemsRs.getString("name"),
                            Money.parse(currency, itemsRs.getDouble("price")),
                            ItemCategory.getCat(itemsRs.getString("category"))
                    );

                    Statement addOnStmt = conn.createStatement();
                    ResultSet addOnRs = addOnStmt.executeQuery("SELECT * FROM addOns WHERE parentID=" + itemsRs.getInt("id") + ";");
                    while (addOnRs.next()){
                        // AddOn(int id, String name, Money price)
                        AddOn tmpAddOn = new AddOn(
                                addOnRs.getInt("id"),
                                addOnRs.getString("name"),
                                Money.parse(currency, addOnRs.getDouble("price"))
                        );
                        tmpItem.addAddOn(tmpAddOn);
                    }
                    rtn.addItem(tmpItem);
                }
            } else {
                throw new RuntimeException("Order: " + id + " does not exist");
            }

            return rtn;
        } catch (SQLException e){
            throw new RuntimeException(e);
        }
    }

    public void recordOrder(Order o){
        updateConnection();
        try {
            int newOrderId = maxID("orders") + 1;
            stmt.executeUpdate("INSERT INTO orders VALUES (" + newOrderId + ", 0, NOW());");

            for (int i = 0; i < o.getItems().size(); i++) {
                int newItemId = maxID("items") + 1;
                Item tmpItem = o.getItems().get(i);
                stmt.executeUpdate("INSERT INTO items VALUES (" + newItemId + ", "
                        + newOrderId + ", '"
                        + tmpItem.getName() + "', "
                        + tmpItem.getPrice().castToDouble() + ", '"
                        + tmpItem.getCategory().toString() + "');");
                for (int j = 0; j < tmpItem.getAddOns().size(); j++) {
                    int newAddOnId = maxID("addOns") + 1;
                    AddOn tmpAddOn = tmpItem.getAddOns().get(j);
                    stmt.executeUpdate("INSERT INTO addOns VALUES (" + newAddOnId + ", "
                            + newItemId + ", '"
                            + tmpAddOn.getName() + "', "
                            + tmpAddOn.getPrice().castToDouble() + ");");
                }
            }

        } catch (SQLException e){
            throw new RuntimeException(e);
        }
    }

    // get maximum ID from a table (basically the number of rows)
    public int maxID(String table) throws SQLException {
        updateConnection();
        try {
            ResultSet rs = stmt.executeQuery("SELECT MAX(id) FROM " + table + ";");
            while (rs.next()) {
                return rs.getInt("MAX(id)");
            }
            return 0;
        } catch (SQLSyntaxErrorException e){
            throw new RuntimeException("Table " + table + " does not exist");
        }
    }

    public void refundOrder(int orderID){
        updateConnection();
        try {
            int len = stmt.executeUpdate("UPDATE orders SET refunded = true WHERE id = " + orderID + ";");
            if(len < 1){
                throw new RuntimeException("Order " + orderID + " does not exist");
            }
        } catch (SQLException e){
            throw new RuntimeException(e);
        }
    }

    public boolean validLogin(String clientID, String passcode) {
        boolean result = false;
        updateConnection();
        try {
            ResultSet rs = stmt.executeQuery("SELECT passcode FROM clients WHERE id = '" + clientID + "';");
            if(rs.next()) {
                if (rs.getString("passcode").equals(passcode)) {
                    result = true;
                }
            }
        } catch (SQLException e){
            throw new RuntimeException(e);
        }

        return result;
    }



    public void closeConnection() throws SQLException {
        conn.close();
    }


    public void terminateConnection() throws SQLException {
        terminated = true;
        conn.close();
        task.cancel();
    }

    @Override
    public String toString(){
        updateConnection();
        String live = "disconnected";
        try {
            if(!conn.isClosed()){
                live = "connected";
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return user + "@" + URL + " status: " + live;
    }
}
